/*
 * Copyright (c) 2008, Felix Opatz
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of  nor the names of its contributors may be used to
 *    endorse or promote products derived from this software without specific
 *    prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 *
 * Date        Change  
 * ---------------------------------------------------------------------------
 * 2008-10-28  Initial implementation.
 */

/* This file contains definitions for the use of a 8250 or compatible UART
 * found in the PC AT and later. The constants are used for direct access
 * to the UART's registers mapped into the I/O memory.
 */

#ifndef UART_H
#define UART_H

/* Base addresses  for the COM ports */
#define UART_BASE_ADDR_COM1    0x3F8
#define UART_BASE_ADDR_COM2    0x2F8
#define UART_BASE_ADDR_COM3    0x3E8
#define UART_BASE_ADDR_COM4    0x2E8

/* Offsets for the UART registers */
#define UART_OFFSET_DATA       0       /* Data Read/Write Register  */
#define UART_OFFSET_IER        1       /* Interrupt Enable Register */
#define UART_OFFSET_IID        2       /* Interrupt ID Register     */
#define UART_OFFSET_LCR        3       /* Line Control Register     */
#define UART_OFFSET_MCR        4       /* Modem Control Register    */
#define UART_OFFSET_LSR        5       /* Line Status Register      */
#define UART_OFFSET_MSR        6       /* Modem Status Register     */
#define UART_OFFSET_DLL        0       /* Divisor Latch Low         */
#define UART_OFFSET_DLH        1       /* Divisor Latch High        */

/* Interrupt Enable Register */
#define UART_IER_RXDI          0       /* RX Data Interrupt         */
#define UART_IER_TXHI          1       /* TX Holding Interrupt      */
#define UART_IER_RXSI          2       /* RX Status Interrupt       */
#define UART_IER_MSCI          3       /* Modem Signal Change Int.  */

/* Line Control Register */
#define UART_LCR_WL            0       /* Word Length, 2 bits       */
#define UART_LCR_WL_5          0x00    /* 5 bits per word           */
#define UART_LCR_WL_6          0x01    /* 6 bits per word           */
#define UART_LCR_WL_7          0x02    /* 7 bits per word           */
#define UART_LCR_WL_8          0x03    /* 8 bits per word           */
#define UART_LCR_STB           2       /* Stop Bits                 */
#define UART_LCR_STB_ONE       0       /* One Stop Bit              */
#define UART_LCR_STB_TWO       1       /* Two Stop Bits             */
#define UART_LCR_PC            3       /* Parity Control, 2 Bits    */
#define UART_LCR_PC_NONE       0x00    /* No parity                 */
#define UART_LCR_PC_ODD        0x01    /* Odd parity                */
#define UART_LCR_PC_EVEN       0x11    /* Even parity               */
#define UART_LCR_SP            5       /* Stick Parity              */
#define UART_LCR_SBR           6       /* Send Break                */
#define UART_LCR_DLAB          7       /* Divisor Latch Access Bit  */

/* Some common baudrate divisors (low latch, high latch = 0) */
#define UART_BDR_2400          0x30    /* 2400   baud               */
#define UART_BDR_4800          0x18    /* 4800   baud               */
#define UART_BDR_9600          0x0C    /* 9600   baud               */
#define UART_BDR_19200         0x06    /* 19200  baud               */
#define UART_BDR_38400         0x03    /* 38400  baud               */
#define UART_BDR_57600         0x02    /* 57600  baud               */
#define UART_BDR_115200        0x01    /* 115200 baud               */
#define UART_MAX_BAUDRATE      115200  /* Baudrate with divisor 1   */

/* Modem Control Register */
#define UART_MCR_DTR           0       /* DTR out                   */
#define UART_MCR_RTS           1       /* RTS out                   */
#define UART_MCR_OUT1          2       /* OUT1, N/C on PC H/W       */
#define UART_MCR_OUT2          3       /* Enable IRQ4 signal        */
#define UART_MCR_LB            4       /* Loopback                  */

/* Line Status Register */
#define UART_LSR_DR            0       /* Data Ready                */
#define UART_LSR_OE            1       /* Overrun Error             */
#define UART_LSR_PE            2       /* Parity Error              */
#define UART_LSR_FE            3       /* Farming Error             */
#define UART_LSR_BR            4       /* Break Received            */
#define UART_LSR_THE           5       /* Tx Holding Register empty */
#define UART_LSR_TSE           6       /* Tx Shift Register empty   */

/* Modem Status Register */
#define UART_MSR_DCTS          0       /* Delta CTS                 */
#define UART_MSR_DDSR          1       /* Delta DSR                 */
#define UART_MSR_RITE          2       /* RI trailing-Edge detect   */
#define UART_MSR_DCD           3       /* Delta CD                  */
#define UART_MSR_CTS           4       /* CTS in status             */
#define UART_MSR_DSR           5       /* DSR in status             */
#define UART_MSR_RI            6       /* RI in status              */
#define UART_MSR_CD            7       /* CD in status              */

#endif /*UART_H*/
